package com.zswl.cloud.springBtach.server.core.controller;

import com.github.microservice.auth.client.content.ResultContent;
import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.auth.security.annotations.ResourceAuth;
import com.github.microservice.auth.security.type.AuthType;
import com.xiaoju.open.oil.commons.OilRequest;
import com.xiaoju.open.oil.core.executor.ResponseDTO;
import com.xiaoju.open.oil.interfaces.domain.AllowancePromotionInfo;
import com.xiaoju.open.oil.interfaces.domain.GenPromotionInfo;
import com.xiaoju.open.oil.interfaces.request.QueryApiOrderListRequest;
import com.xiaoju.open.oil.interfaces.request.QueryGetPromotionRequest;
import com.xiaoju.open.oil.interfaces.response.*;
import com.zswl.cloud.springBatch.client.model.request.*;
import com.zswl.cloud.springBtach.server.core.domain.XiaoJuStore;
import com.zswl.cloud.springBtach.server.core.service.XiaoJuService;
import io.swagger.annotations.ApiOperation;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.*;

import java.util.List;

@Log4j2
@RestController
@RequestMapping("xiaoju")
public class XiaoJuController {

    @Autowired
    XiaoJuService xiaoJuService;

    @ApiOperation("推送-同步订单信息")
    @RequestMapping(value = "notifyOrderInfo", method = RequestMethod.POST)
    public ResponseDTO notifyOrderInfo(@RequestBody OilRequest oilRequest) {

        return xiaoJuService.notifyOrderInfo(oilRequest);
    }

    @ApiOperation("推送-同步加油站信息")
    @RequestMapping(value = "receivingGasStation", method = RequestMethod.POST)
    public ResponseDTO receivingGasStation(@RequestBody OilRequest oilRequest) {

        return xiaoJuService.notificationPushStoreInfoRequest(oilRequest);
    }

    @ApiOperation("查看附近的加油站信息")
    @ResourceAuth(value = "user", type = AuthType.User)
    @RequestMapping(value = "nearbyStore", method = RequestMethod.POST)
    public ResultContent<List<XiaoJuStoreModel>> nearbyStore(@RequestBody NearbyModel nearbyModel) {
        return xiaoJuService.nearbyStore(nearbyModel);
    }

    @ApiOperation("查询门店价格")
    @ResourceAuth(value = "user", type = AuthType.User)
    @RequestMapping(value = "queryStorePrice", method = RequestMethod.POST)
    public ResultContent<List<QueryApiStorePriceResponse>> queryStorePrice(@RequestBody List<String> storeIdList) {
        return xiaoJuService.queryStorePrice(storeIdList);
    }

    @ApiOperation("查看加油站详情")
    @RequestMapping(value = "queryStoreDetail/{storeId}", method = RequestMethod.GET)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<XiaoJuStore> queryStoreDetail(@PathVariable("storeId") String storeId) {

        return xiaoJuService.queryStoreDetail(storeId);
    }

    @ApiOperation("查看加油站详情2-调用小橘接口")
    @RequestMapping(value = "queryStoreDetail2/{storeId}", method = RequestMethod.GET)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<QueryStoreDetailResponse> queryStoreDetail2(@PathVariable("storeId") String storeId) {

        return xiaoJuService.queryStoreDetail2(storeId);
    }

    @ApiOperation("价格计算")
    @RequestMapping(value = "queryCalPrice", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<QueryCalPriceResponse> queryCalPrice(@RequestBody QueryCalPriceModel queryCalPriceModel) {
        return xiaoJuService.queryCalPrice(queryCalPriceModel);
    }

    @ApiOperation("批量价格计算")
    @RequestMapping(value = "queryCalPrices", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<QueryCalBatchPriceResponse> queryCalPrices(@RequestBody QueryCalPricesModel queryCalPriceModel) {
        return xiaoJuService.queryCalPrices(queryCalPriceModel);
    }

    @ApiOperation("创建订单")
    @RequestMapping(value = "queryNewOrder", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<QueryNewOrderResponse> queryNewOrder(@RequestBody XiaoJuQueryNewOrderModel xiaoJuQueryNewOrderModel) {
        return xiaoJuService.queryNewOrder(xiaoJuQueryNewOrderModel);
    }

    @ApiOperation("取消订单")
    @RequestMapping(value = "cancelOrder", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent cancelOrder(@RequestBody QueryNewOrderResponse queryNewOrderResponse) {
        xiaoJuService.cancelOrder(queryNewOrderResponse.getOutOrderId(), queryNewOrderResponse.getXjOrderId());
        return ResultContent.build(ResultState.Success);
    }
    @ApiOperation("查询订单列表")
    @RequestMapping(value = "queryOrderList", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent queryOrderList(@RequestBody QueryApiOrderListRequest queryApiOrderListRequest) {
        xiaoJuService.queryOrderList(queryApiOrderListRequest);
        return ResultContent.build(ResultState.Success);
    }

    @ApiOperation("下单前查询最优可用券")
    @RequestMapping(value = "queryOrderPromotionList", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<List<AllowancePromotionInfo>> queryOrderPromotionList(@RequestBody XiaoJuQueryOrderPromotionListModel xiaoJuQueryOrderPromotionListModel) {
        ResultContent<List<AllowancePromotionInfo>> listResultContent = xiaoJuService.queryOrderPromotionList(xiaoJuQueryOrderPromotionListModel);
        return ResultContent.buildContent(listResultContent);
    }

    @ApiOperation("查询多个门店可用券")
    @RequestMapping(value = "queryStorePromotionList", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<List<AllowancePromotionInfo>> queryStorePromotionList(@RequestBody List<String> storeIdList) {
        ResultContent<List<AllowancePromotionInfo>> listResultContent = xiaoJuService.queryStorePromotionList(storeIdList);
        return ResultContent.buildContent(listResultContent);
    }

    @ApiOperation("查询用户所有券列表")
    @RequestMapping(value = "queryUserPromotionList", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<QueryUserPromotionListResponse> queryUserPromotionList(@RequestBody XiaoJuQueryUserPromotionListModel xiaoJuQueryUserPromotionListModel) {
        ResultContent<QueryUserPromotionListResponse> listResultContent = xiaoJuService.queryUserPromotionList(xiaoJuQueryUserPromotionListModel);
        return ResultContent.buildContent(listResultContent);
    }

    @ApiOperation("券发放")
    @RequestMapping(value = "getPromotion", method = RequestMethod.POST)
    @ResourceAuth(value = "user", type = AuthType.User)
    public ResultContent<List<GenPromotionInfo>> getPromotion(@RequestBody QueryGetPromotionRequest queryGetPromotionRequest) {
        ResultContent<List<GenPromotionInfo>> listResultContent = xiaoJuService.getPromotion(queryGetPromotionRequest);
        return ResultContent.buildContent(listResultContent);
    }


}
