package com.zswl.cloud.springBtach.server.core.job.video;

import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.github.microservice.auth.client.content.ResultContent;
import com.github.microservice.auth.client.content.ResultState;
import com.zhongshu.payment.client.model.order.v2.GoodsResultModel;
import com.zhongshu.payment.client.model.order.v2.VerificationModel;
import com.zhongshu.payment.client.service.v2.OrderService2;
import com.zhongshu.payment.client.service.v2.PaymentService2;
import com.zswl.cloud.shop.client.dto.yqd.OrderDto;
import com.zswl.cloud.shop.client.service.GoodsService;
import com.zswl.cloud.springBtach.server.core.api.video.YqdServiceImpl;
import com.zswl.cloud.springBtach.server.core.job.standard.StandarDeliveryJob;
import lombok.Data;
import lombok.experimental.Delegate;
import lombok.extern.log4j.Log4j2;
import org.springframework.batch.core.Job;
import org.springframework.batch.core.Step;
import org.springframework.batch.core.launch.support.RunIdIncrementer;
import org.springframework.batch.repeat.RepeatStatus;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;

@Data
@Log4j2
@Configuration
public class VideoDeliveryJob {

    @Autowired
    @Delegate
    StandarDeliveryJob standarDeliveryJob;

    @Autowired
    GoodsService goodsService;

    @Autowired
    OrderService2 orderService2;

    @Autowired
    PaymentService2 paymentService2;

    @Autowired
    YqdServiceImpl yqdServiceImpl;


    public Step videoDeliveryStep() {
        return getStepBuilderFactory().get("videoDeliveryStep")
                .tasklet((contribution, chunkContext) -> {
                    GoodsResultModel goodsResultModel = getParameter().get();

                    JSONObject entries = JSONUtil.parseObj(goodsResultModel.getExtend());
                    String account = String.valueOf(entries.get("account"));


                    VerificationModel verificationModel = new VerificationModel();
                    verificationModel.setId(goodsResultModel.getId());
                    verificationModel.setShopId(goodsResultModel.getShopId());
                    verificationModel.setVerification(new JSONObject());
                    ResultContent verification = orderService2.verification(verificationModel);
                    if (!verification.getState().equals(ResultState.Success)){

                        log.info("核销状态变更失败：{}",JSONUtil.toJsonStr(goodsResultModel));
                        getResultContent().set(ResultContent.build(ResultState.Fail));
                        return RepeatStatus.FINISHED;

                    }
                    OrderDto orderDto = new OrderDto();
                    orderDto.setCommodityId(goodsResultModel.getGoodsId());
                    orderDto.setPhone(account);
                    ResultContent resultContent = yqdServiceImpl.create(orderDto);
                    log.info("视频会员响应结果：{}", resultContent.getContent());
                    if (!resultContent.getState().equals(ResultState.Success)) {
                        getResultContent().set(ResultContent.build(ResultState.Fail));
                        return RepeatStatus.FINISHED;
                    }
                    String extend = goodsResultModel.getExtend();
                    JSONObject entries1 = JSONUtil.parseObj(extend);
                    entries1.set("response", resultContent.getContent());
                    orderService2.updateExtend(goodsResultModel.getId(), JSONUtil.toJsonStr(entries1));
                    getResultContent().set(ResultContent.build(ResultState.Success));
                    return RepeatStatus.FINISHED;

                })
                .build();
    }

    @Bean
    public Job videoDelivery() {
        return getJobBuilderFactory()
                .get("videoDelivery")
                .start(videoDeliveryStep())
                .incrementer(new RunIdIncrementer())
                .build();
    }

}

