package com.zswl.cloud.springBtach.server.core.service;


import cn.hutool.core.lang.Snowflake;
import cn.hutool.core.util.ObjectUtil;
import cn.hutool.json.JSONUtil;
import com.github.microservice.auth.client.content.ResultContent;
import com.github.microservice.auth.client.content.ResultState;
import com.github.microservice.auth.security.helper.AuthHelper;
import com.github.microservice.auth.security.model.AuthDetails;
import com.xiaoju.open.oil.commons.OilRequest;
import com.xiaoju.open.oil.commons.config.OilConfig;
import com.xiaoju.open.oil.core.QueryClient;
import com.xiaoju.open.oil.core.QueryClientFactory;
import com.xiaoju.open.oil.core.executor.ResponseDTO;
import com.xiaoju.open.oil.interfaces.domain.*;
import com.xiaoju.open.oil.interfaces.request.*;
import com.xiaoju.open.oil.interfaces.request.miniprogram.MiniCancelOrderStatusRequest;
import com.xiaoju.open.oil.interfaces.response.*;
import com.xiaoju.open.oil.security.secret.SecurityService;
import com.zswl.cloud.springBatch.client.model.request.*;
import com.zswl.cloud.springBtach.server.core.dao.ApiLogDao;
import com.zswl.cloud.springBtach.server.core.dao.XiaoJuOrderDao;
import com.zswl.cloud.springBtach.server.core.dao.XiaoJuStoreDao;
import com.zswl.cloud.springBtach.server.core.domain.ApiLog;
import com.zswl.cloud.springBtach.server.core.domain.XiaoJuOrder;
import com.zswl.cloud.springBtach.server.core.domain.XiaoJuStore;
import lombok.extern.log4j.Log4j2;
import org.springframework.beans.BeanUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.ArrayList;
import java.util.List;

@Log4j2
@Service
public class XiaoJuService {


    @Autowired
    AuthHelper authHelper;

    @Autowired
    XiaoJuStoreDao xiaoJuStoreDao;

    @Autowired
    OilConfig oilConfig;

    @Autowired
    XiaoJuOrderDao xiaoJuOrderDao;

    @Autowired
    Snowflake snowflake;

    @Autowired
    ApiLogDao apiLogDao;

    // 查询加油站详情
    public ResponseDTO notificationPushStoreInfoRequest(OilRequest oilRequest) {


        log.info("原始请求：{}", JSONUtil.toJsonStr(oilRequest));

        String s = SecurityService.genDecodeData(oilRequest.getData(), oilConfig.getDataSecret(), oilConfig.getDataSecretIV());


        ApiLog apiLog = new ApiLog();
        apiLog.setService("notificationPushStoreInfoRequest");
        apiLog.setRequest(s);
        apiLog.setSystem("xiaoju");


        XiaoJuStore xiaoJuStore = JSONUtil.toBean(s, XiaoJuStore.class);

        XiaoJuStore byStoreId = xiaoJuStoreDao.findByStoreId(xiaoJuStore.getStoreId());
        if (byStoreId != null) {
            BeanUtils.copyProperties(byStoreId, xiaoJuStore, "id", "updateTime");
            byStoreId.setLocation(new double[]{xiaoJuStore.getLon(), xiaoJuStore.getLat()});
            xiaoJuStoreDao.save(byStoreId);
        } else {
            xiaoJuStore.setLocation(new double[]{xiaoJuStore.getLon(), xiaoJuStore.getLat()});
            xiaoJuStoreDao.save(xiaoJuStore);
        }


        String data = SecurityService.genEncodeData("{\"success\":true,\"msg\":\"ok\"}", oilConfig.getDataSecret(), oilConfig.getDataSecretIV());
        String sig = SecurityService.genSigData(data, oilConfig.getDataSecret());
        ResponseDTO responseDTO = new ResponseDTO();
        responseDTO.setAppKey(oilConfig.getAppId());
        responseDTO.setCode("0");
        responseDTO.setMsg("请求成功");
        responseDTO.setData(data);
        responseDTO.setSig(sig);
        apiLog.setResponse(JSONUtil.toJsonStr(responseDTO));
        apiLogDao.save(apiLog);
        return responseDTO;

    }

    public ResponseDTO notifyOrderInfo(OilRequest oilRequest) {


        String s = SecurityService.genDecodeData(oilRequest.getData(), oilConfig.getDataSecret(), oilConfig.getDataSecretIV());

        ApiLog apiLog = new ApiLog();
        apiLog.setService("notifyOrderInfo");
        apiLog.setRequest(s);
        apiLog.setSystem("xiaoju");

        log.info("通知 加油站详情：{}", s);
        NotificationReceiveRefundRequest xiaoJuStore = JSONUtil.toBean(s, NotificationReceiveRefundRequest.class);

        XiaoJuOrder byOutOrderId = xiaoJuOrderDao.findByOutOrderId(xiaoJuStore.getOutOrderId());
        BeanUtils.copyProperties(byOutOrderId, xiaoJuStore, "outOrderId");
        xiaoJuOrderDao.save(byOutOrderId);

        String data = SecurityService.genEncodeData("{\"success\":true,\"msg\":\"ok\"}", oilConfig.getDataSecret(), oilConfig.getDataSecretIV());
        String sig = SecurityService.genSigData(data, oilConfig.getDataSecret());
        ResponseDTO responseDTO = new ResponseDTO();
        responseDTO.setAppKey(oilConfig.getAppId());
        responseDTO.setCode("0");
        responseDTO.setMsg("请求成功");
        responseDTO.setData(data);
        responseDTO.setSig(sig);
        apiLog.setResponse(JSONUtil.toJsonStr(responseDTO));
        apiLogDao.save(apiLog);
        return responseDTO;

    }

    public ResultContent<XiaoJuStore> queryStoreDetail(String storeId) {
        XiaoJuStore byStoreId = xiaoJuStoreDao.findByStoreId(storeId);
        return ResultContent.build(ResultState.Success, byStoreId);
    }

    public ResultContent<QueryStoreDetailResponse> queryStoreDetail2(String storeId) {
        AuthDetails currentUser = authHelper.getCurrentUser();
        QueryStoreDetailRequest queryStoreDetailRequest = new QueryStoreDetailRequest();

        queryStoreDetailRequest.setOutUserId(currentUser.getUserId());
        queryStoreDetailRequest.setMobile(currentUser.getPhone());
        queryStoreDetailRequest.setStoreId(storeId);
        queryStoreDetailRequest.setOpenChannel(1);

        QueryClient queryClient = QueryClientFactory.create(oilConfig);
        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryStoreDetail");
        apiLog.setRequest(JSONUtil.toJsonStr(queryStoreDetailRequest));
        apiLog.setSystem("xiaoju");
        try {
            QueryStoreDetailResponse queryStoreDetailResponse = queryClient.api().queryStoreDetail(queryStoreDetailRequest);

            XiaoJuStore byStoreId = xiaoJuStoreDao.findByStoreId(storeId);
            List<StoreDetailItemInfo> itemInfoList = queryStoreDetailResponse.getItemInfoList();
            if (ObjectUtil.isEmpty(byStoreId)) {
                XiaoJuStore xiaoJuStore = new XiaoJuStore();
                BeanUtils.copyProperties(queryStoreDetailResponse, xiaoJuStore);
                xiaoJuStore.setLogo(queryStoreDetailResponse.getStoreLogo());
                xiaoJuStore.setLocation(new double[]{queryStoreDetailResponse.getLon(), queryStoreDetailResponse.getLat()});
                xiaoJuStore.setPicList(queryStoreDetailResponse.getStorePicList());


                List<ItemPriceInfoForCallback> itemInfoList1 = new ArrayList<>();
                for (StoreDetailItemInfo storeDetailItemInfo : itemInfoList) {
                    ItemPriceInfoForCallback itemPriceInfoForCallback = new ItemPriceInfoForCallback();
                    BeanUtils.copyProperties(storeDetailItemInfo, itemPriceInfoForCallback);
                    itemPriceInfoForCallback.setGunNoList(storeDetailItemInfo.getGunNos());
                    itemInfoList1.add(itemPriceInfoForCallback);
                }
                xiaoJuStore.setItemInfoList(itemInfoList1);
                xiaoJuStoreDao.save(xiaoJuStore);
            } else {

                BeanUtils.copyProperties(queryStoreDetailResponse, byStoreId, "id", "updateTime");
                byStoreId.setLocation(new double[]{queryStoreDetailResponse.getLon(), queryStoreDetailResponse.getLat()});
                byStoreId.setLogo(queryStoreDetailResponse.getStoreLogo());
                byStoreId.setPicList(queryStoreDetailResponse.getStorePicList());
                List<ItemPriceInfoForCallback> itemInfoList1 = new ArrayList<>();
                for (StoreDetailItemInfo storeDetailItemInfo : itemInfoList) {
                    ItemPriceInfoForCallback itemPriceInfoForCallback = new ItemPriceInfoForCallback();
                    BeanUtils.copyProperties(storeDetailItemInfo, itemPriceInfoForCallback);
                    itemPriceInfoForCallback.setGunNoList(storeDetailItemInfo.getGunNos());
                    itemInfoList1.add(itemPriceInfoForCallback);
                }
                byStoreId.setItemInfoList(itemInfoList1);
                xiaoJuStoreDao.save(byStoreId);


            }

            apiLog.setResponse(JSONUtil.toJsonStr(queryStoreDetailResponse));
            apiLogDao.save(apiLog);


            return ResultContent.buildContent(queryStoreDetailResponse);
        } catch (Exception e) {

            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }

    }

    public ResultContent<List<QueryApiStorePriceResponse>> queryStorePrice(List<String> storeIdList) {
        String userId = authHelper.getCurrentUser().getUserId();
        QueryApiStorePriceRequest queryApiStorePriceRequest = new QueryApiStorePriceRequest();
        queryApiStorePriceRequest.setOpenChannel(1);
        queryApiStorePriceRequest.setOutUserId(userId);
        queryApiStorePriceRequest.setStoreIds(storeIdList);


        QueryClient queryClient = QueryClientFactory.create(oilConfig);
        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryStorePrice");
        apiLog.setRequest(JSONUtil.toJsonStr(queryApiStorePriceRequest));
        apiLog.setSystem("xiaoju");
        try {
            List<QueryApiStorePriceResponse> list1 = queryClient.api().queryStorePrice(queryApiStorePriceRequest);
            apiLog.setResponse(JSONUtil.toJsonStr(list1));
            apiLogDao.save(apiLog);
            return ResultContent.buildContent(list1);
        } catch (Exception e) {
            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }
    }


    public ResultContent<List<XiaoJuStoreModel>> nearbyStore(NearbyModel nearbyModel) {

        List<XiaoJuStoreModel> nearbyTargets = xiaoJuStoreDao.findNearbyTargets(nearbyModel);
        return ResultContent.buildContent(nearbyTargets);
    }


    // 价格计算
    public ResultContent<QueryCalPriceResponse> queryCalPrice(QueryCalPriceModel queryCalPriceModel) {

        String userId = authHelper.getCurrentUser().getUserId();
        QueryCalPriceRequest queryCalPriceRequest = new QueryCalPriceRequest();
        queryCalPriceRequest.setStoreId(queryCalPriceModel.getStoreId());
        queryCalPriceRequest.setOpenChannel(1);
        queryCalPriceRequest.setOutUserId(userId);
        queryCalPriceRequest.setItemName(queryCalPriceModel.getItemName());
        queryCalPriceRequest.setAmount(queryCalPriceModel.getAmount());

        queryCalPriceRequest.setPromotionInfo(queryCalPriceModel.getAllowanceinfo());

        QueryClient queryClient = QueryClientFactory.create(oilConfig);
        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryCalPrice");
        apiLog.setRequest(JSONUtil.toJsonStr(queryCalPriceRequest));
        apiLog.setSystem("xiaoju");
        try {
            QueryCalPriceResponse queryCalPriceResponse = queryClient.api().queryCalPrice(queryCalPriceRequest);
            apiLog.setResponse(JSONUtil.toJsonStr(queryCalPriceResponse));
            apiLogDao.save(apiLog);
            return ResultContent.buildContent(queryCalPriceResponse);
        } catch (Exception e) {
            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }

    }


    // 批量价格计算
    public ResultContent<QueryCalBatchPriceResponse> queryCalPrices(QueryCalPricesModel queryCalPricesModel) {

        String userId = authHelper.getCurrentUser().getUserId();
        QueryCalBatchPriceRequest queryCalPriceRequest = new QueryCalBatchPriceRequest();
        queryCalPriceRequest.setStoreId(queryCalPricesModel.getStoreId());
        queryCalPriceRequest.setOpenChannel(1);
        queryCalPriceRequest.setOutUserId(userId);

        queryCalPriceRequest.setItemList(queryCalPricesModel.getItemList());

        QueryClient queryClient = QueryClientFactory.create(oilConfig);
        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryCalPrices");
        apiLog.setRequest(JSONUtil.toJsonStr(queryCalPriceRequest));
        apiLog.setSystem("xiaoju");
        try {
            QueryCalBatchPriceResponse queryCalBatchPriceResponse = queryClient.api().queryCalPrices(queryCalPriceRequest);
            apiLog.setResponse(JSONUtil.toJsonStr(queryCalBatchPriceResponse));
            apiLogDao.save(apiLog);
            return ResultContent.buildContent(queryCalBatchPriceResponse);
        } catch (Exception e) {
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }
    }


    // 创建订单
    public ResultContent<QueryNewOrderResponse> queryNewOrder(XiaoJuQueryNewOrderModel xiaoJuQueryNewOrderModel) {
        AuthDetails currentUser = authHelper.getCurrentUser();
        String phone = currentUser.getPhone();
        String userId = currentUser.getUserId();

        XiaoJuStore byStoreId = xiaoJuStoreDao.findByStoreId(xiaoJuQueryNewOrderModel.getStoreId());

        String outOrderId = snowflake.nextIdStr();
        QueryNewOrderRequest queryNewOrderRequest = new QueryNewOrderRequest();
        queryNewOrderRequest.setOutUserId(userId);
        queryNewOrderRequest.setMobile(phone);
        queryNewOrderRequest.setOutOrderId(outOrderId);
        queryNewOrderRequest.setLat(byStoreId.getLat());
        queryNewOrderRequest.setLon(byStoreId.getLon());
        queryNewOrderRequest.setStoreId(byStoreId.getStoreId());
        queryNewOrderRequest.setItemId(xiaoJuQueryNewOrderModel.getItemId());
        queryNewOrderRequest.setGunNo(xiaoJuQueryNewOrderModel.getGunNo());
        queryNewOrderRequest.setOriginalAmount(xiaoJuQueryNewOrderModel.getOriginalAmount());
        queryNewOrderRequest.setPaymentAmount(xiaoJuQueryNewOrderModel.getPaymentAmount());
        queryNewOrderRequest.setServiceFee(xiaoJuQueryNewOrderModel.getServiceFee());
        queryNewOrderRequest.setLitre(xiaoJuQueryNewOrderModel.getLitre());
        queryNewOrderRequest.setVipPrice(xiaoJuQueryNewOrderModel.getVipPrice());
        queryNewOrderRequest.setStorePrice(xiaoJuQueryNewOrderModel.getStorePrice());
        queryNewOrderRequest.setPromotionInfo(xiaoJuQueryNewOrderModel.getPromotionInfo());

        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryNewOrder");
        apiLog.setRequest(JSONUtil.toJsonStr(queryNewOrderRequest));
        apiLog.setSystem("xiaoju");
        try {
            QueryNewOrderResponse queryNewOrderResponse = queryClient.api().queryNewOrder(queryNewOrderRequest);


            XiaoJuOrder xiaoJuOrder = new XiaoJuOrder();
            BeanUtils.copyProperties(queryNewOrderResponse, xiaoJuOrder);
            BeanUtils.copyProperties(queryNewOrderRequest, xiaoJuOrder);

            xiaoJuOrderDao.save(xiaoJuOrder);


            apiLog.setResponse(JSONUtil.toJsonStr(queryNewOrderResponse));
            apiLogDao.save(apiLog);

            return ResultContent.buildContent(queryNewOrderResponse);
        } catch (Exception e) {

            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());

        }


    }


    // 取消订单
    public void cancelOrder(String orderNo,String xjOrderId) {
        MiniCancelOrderStatusRequest cancelOrderStatusRequest = new MiniCancelOrderStatusRequest();
        cancelOrderStatusRequest.setOutOrderId(orderNo);
        cancelOrderStatusRequest.setXjOrderId(xjOrderId);
        QueryClient queryClient = QueryClientFactory.create(oilConfig);
        queryClient.apiMini().cancelOrder(cancelOrderStatusRequest);
        ApiLog apiLog = new ApiLog();
        apiLog.setService("cancelOrder");
        apiLog.setRequest(JSONUtil.toJsonStr(cancelOrderStatusRequest));
        apiLog.setSystem("xiaoju");
        apiLog.setResponse(null);
        apiLogDao.save(apiLog);

    }



    // 同步订单状态
    public  void syncOrderStatus(SynOrderStatusRequest synOrderStatusRequest) {

        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        queryClient.api().syncOrderStatus(synOrderStatusRequest);

    }


    // 查询订单列表
    public QueryApiOrderListResponse queryOrderList(QueryApiOrderListRequest queryApiOrderListRequest) {
        String userId = authHelper.getCurrentUser().getUserId();

        queryApiOrderListRequest.setMobile(authHelper.getCurrentUser().getPhone());
        queryApiOrderListRequest.setOutUserId(userId);

        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        QueryApiOrderListResponse queryApiOrderListResponse = queryClient.api().queryOrderList(queryApiOrderListRequest);

        return queryApiOrderListResponse;


    }


    // 下单前查询最优可用券
    public ResultContent<List<AllowancePromotionInfo>> queryOrderPromotionList(XiaoJuQueryOrderPromotionListModel xiaoJuQueryOrderPromotionListModel) {
        String userId = authHelper.getCurrentUser().getUserId();
        QueryOrderPromotionListRequest request = new QueryOrderPromotionListRequest();
        request.setOutUserId(userId);
        request.setStoreId(xiaoJuQueryOrderPromotionListModel.getStoreId());
        request.setAllowanceType(2);
        request.setMobile(authHelper.getCurrentUser().getPhone());
        request.setOrderAmount(xiaoJuQueryOrderPromotionListModel.getOrderAmount());

        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryOrderPromotionList");
        apiLog.setRequest(JSONUtil.toJsonStr(request));
        apiLog.setSystem("xiaoju");
        try {
            List<AllowancePromotionInfo> response = queryClient.promotion().queryOrderPromotionList(request);
            apiLog.setResponse(JSONUtil.toJsonStr(response));
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Success, response);
        } catch (Exception e) {
            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }

    }


    // 查询多个门店可用券
    public ResultContent<List<AllowancePromotionInfo>> queryStorePromotionList(List<String> storeIdList) {
        AuthDetails currentUser = authHelper.getCurrentUser();
        QueryStorePromotionListRequest request = new QueryStorePromotionListRequest();
        request.setMobile(currentUser.getPhone());
        request.setAllowanceType(2);
        request.setOutUserId(currentUser.getUserId());
        request.setStoreIdList(storeIdList);

        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryStorePromotionList");
        apiLog.setRequest(JSONUtil.toJsonStr(request));
        apiLog.setSystem("xiaoju");
        try {
            List<AllowancePromotionInfo> response = queryClient.promotion().queryStorePromotionList(request);
            apiLog.setResponse(JSONUtil.toJsonStr(response));
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Success, response);
        } catch (Exception e) {
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }

    }

    // 查询用户所有券列表
    public ResultContent<QueryUserPromotionListResponse> queryUserPromotionList(XiaoJuQueryUserPromotionListModel xiaoJuQueryUserPromotionListModel) {
        AuthDetails currentUser = authHelper.getCurrentUser();
        QueryUserPromotionListRequest request = new QueryUserPromotionListRequest();
        request.setMobile(currentUser.getPhone());
        request.setAllowanceType(2);
        request.setOutUserId(currentUser.getUserId());
        request.setStatus(xiaoJuQueryUserPromotionListModel.getStatus());
        request.setPageIndex(xiaoJuQueryUserPromotionListModel.getPageIndex());
        request.setPageSize(xiaoJuQueryUserPromotionListModel.getPageSize());

        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryUserPromotionList");
        apiLog.setRequest(JSONUtil.toJsonStr(request));
        apiLog.setSystem("xiaoju");
        try {
            QueryUserPromotionListResponse response = queryClient.promotion().queryUserPromotionList(request);
            apiLog.setResponse(JSONUtil.toJsonStr(response));
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Success, response);
        } catch (Exception e) {

            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());
        }
    }

    // 券发放
    public ResultContent<List<GenPromotionInfo>> getPromotion(QueryGetPromotionRequest queryGetPromotionRequest) {
        QueryClient queryClient = QueryClientFactory.create(oilConfig);

        ApiLog apiLog = new ApiLog();
        apiLog.setService("queryUserPromotionList");
        apiLog.setRequest(JSONUtil.toJsonStr(queryGetPromotionRequest));
        apiLog.setSystem("xiaoju");
        try {
            List<GenPromotionInfo> response = queryClient.promotion().getPromotion(queryGetPromotionRequest);
            apiLog.setResponse(JSONUtil.toJsonStr(response));
            apiLogDao.save(apiLog);
            return ResultContent.buildContent(response);
        } catch (Exception e) {
            e.printStackTrace();
            apiLog.setResponse(e.getCause().getMessage());
            apiLogDao.save(apiLog);
            return ResultContent.build(ResultState.Fail, e.getCause().getMessage());

        }
    }
}
